%bcond_with shared_libuv
%bcond_with shared_cares
%bcond_with shared_brotli
%bcond_with shared_icu

%define major 18
%define minor 18
%define fix 2

%define node_prefix %{_libdir}/%{name}

Name: nodejs%{major}
Version: %{major}.%{minor}.%{fix}
Release: 1
License: MIT
Summary: Node.js JavaScript Runtime
Url: https://www.nodejs.org

BuildRequires: pkg-config
BuildRequires: python39
BuildRequires: zlib-devel
BuildRequires: openssl-1.1-devel
%{?with_shared_icu:BuildRequires: icu-devel}
%{?with_shared_cares:BuildRequires: c-ares-devel}
%{?with_shared_libuv:BuildRequires: libuv-devel}
%{?with_shared_brotli:BuildRequires: brotli-devel}

BuildRequires: gcc10
BuildRequires: gcc10-cplusplus
BuildRequires: libstdcplusplus6-10-devel >= 10.3.0

Requires: libopenssl1_1 >= 1.1.1-6
Requires: python39
Requires: update-alternatives

Requires: libstdcplusplus6 >= 10.3.0
Requires: libgcc_s1 >= 10.3.0

Requires(post): update-alternatives
Requires(postun): update-alternatives

Conflicts: nodever < 1.0.0
Source0: https://nodejs.org/dist/v%{version}/node-v%{version}.tar.gz

Patch6: nodejs18-deps_v8.patch

%description
Node.js is a JavaScript runtime built on Chrome V8 JavaScript engine. Node.js
uses an event-driven, non-blocking I/O model that makes it lightweight and
efficient.

%prep
%setup -q -n node-v%{version}

%patch6 -p1

# Fix the wrong dependency /usr/bin/python
find deps tools ./configure* -type f | xargs perl -pi -e 's|/usr/bin/python|%{_bindir}/python3.9|g'

%build

export CC=gcc-10
export CXX=g++-10

./configure \
  --prefix=%{node_prefix} \
  --dest-cpu=ppc64 \
  --without-node-snapshot \
  --shared-openssl \
  --shared-zlib \
   %{?with_shared_icu:--with-intl=system-icu} \
   %{?with_shared_cares:--shared-cares} \
   %{?with_shared_libuv:--shared-libuv} \
   %{?with_shared_brotli:--shared-brotli} \
  # end

%make_build
# basic verification
./node -e "console.log(process.versions)"

%check

make test-only

%install

%make_install V=1 PORTABLE=1

# Create node headers tar file for use with npm
# otherwise, npm will download and use the unpatched versions :(
#
# NOTE: This can be removed once npm changes have been merged upstream.
cd %{buildroot}%{node_prefix}
tar -czf node-headers.tar.gz ./include
 
# Create an "npm builtin config file" to point npm at the header tarball
# created above. If users want to override, they can do so using the per-user,
# or per-project config files:
# https://docs.npmjs.com/cli/v8/configuring-npm/npmrc#files
mkdir ./etc
cat <<EOF > ./etc/npmrc
tarball=%{node_prefix}/node-headers.tar.gz
script-shell=%{_bindir}/bash
EOF

%post
###############################################################################
# processing after the install
# $1 = 1: install
# $1 = 2+: upgrade
###############################################################################

type=upgrade
[ $1 -eq 1 ] && type=install

if [ $type = 'install' ]
then
  # set up the just installed version of Node.js in update-alternatives
  %{_bindir}/alternatives --install %{_bindir}/node node %{_libdir}/nodejs%{major}/bin/node %{major} \
  --slave %{_bindir}/npm npm %{_libdir}/nodejs%{major}/bin/npm \
  --slave %{_bindir}/npx npx %{_libdir}/nodejs%{major}/bin/npx
fi


%postun
###############################################################################
# processing after the uninstall
# $1 = 0: uninstall
# $1 = 1: upgrade
###############################################################################

type=uninstall
[ $1 -eq 1 ] && type=upgrade

# After uninstallation, need to check if it was truly an uninstallation, and if
# so, remove the the update-alternatives entry for this version of node
if [ $type = 'uninstall' ]
then
  %{_bindir}/alternatives --remove node %{_libdir}/nodejs%{major}/bin/node
fi


%files
# Set the default owner to QSYS and group to *NONE
%defattr(-, qsys, *none)

%{node_prefix}

%changelog
* Fri Oct 13 2023 Abdirahim Musse <abdirahim.musse@ibm.com> - 18.18.2
- Update for security release
- https://github.com/nodejs/node/releases/tag/v18.18.2
- https://nodejs.org/en/blog/vulnerability/october-2023-security-releases

* Wed Aug 9 2023 Abdirahim Musse <abdirahim.musse@ibm.com> - 18.17.1
- Update for security release
- https://github.com/nodejs/node/releases/tag/v18.17.1
- https://nodejs.org/en/blog/vulnerability/august-2023-security-releases

* Mon Jul 24 2023 Abdirahim Musse <abdirahim.musse@ibm.com> - 18.17.0
- Update to 18.17.0
- https://github.com/nodejs/node/releases/tag/v18.17.0

* Tue Jun 20 2023 Abdirahim Musse <abdirahim.musse@ibm.com> - 18.16.1
- Update for security release
- https://nodejs.org/en/blog/vulnerability/june-2023-security-releases
- Drop patches that got merged upstream
- https://github.com/nodejs/node/commit/4b25b98bd872b84859f0f89ede83b9ba3138d556

* Fri Feb 17 2023 Xu Meng <mengxumx@cn.ibm.com> - 18.14.1
- update to address below vulnerabilities -
- CVE-2023-23918
- CVE-2023-23919
- CVE-2023-23936
- CVE-2023-24807
- CVE-2023-23920

* Thu Feb 09 2023 Kevin Adler <kadler@us.ibm.com> - 18.12.1
- BuildRequire openssl-1.1-devel instead of openssl-devel

* Wed Nov 9 2022 Xu Meng <mengxumx@cn.ibm.com> - 18.12.1
- update to address below vulnerabilities -
- CVE-2022-3602
- CVE-2022-3786
- CVE-2022-43548

* Wed Sep 28 2022 Xu Meng <mengxumx@cn.ibm.com> - 18.9.1
- update to address below vulnerabilities -
- CVE-2022-32212
- CVE-2022-32215
- CVE-2022-32213
- CVE-2022-35256
- CVE-2022-32222
- CVE-2022-35255

* Thu Aug 25 2022 Xu Meng <mengxumx@cn.ibm.com> - 18.5.0
- set proper script shell for running npm

* Fri Jul 8 2022 Xu Meng <mengxumx@cn.ibm.com> - 18.5.0
- update to address below vulnerabilities -
- CVE-2022-32213
- CVE-2022-32214
- CVE-2022-32215
- CVE-2022-32212
- CVE-2022-32222

* Fri Apr 22 2022 Kevin Adler <kadler@us.ibm.com> - 18.0.0
- Update patch for npm to set OS correctly under Python 3.9
- Move npmrc to node_prefix/etc

* Wed Apr 20 2022 Xu Meng <mengxumx@cn.ibm.com> - 18.0.0
- Update to v18.0.0

* Wed Feb 16 2022 Xu Meng <mengxumx@cn.ibm.com>
- Initial package for verification
